@@ -5,12 +5,13 @@
 /*--------------------------------------------------------------------*/
 
 /*
-   This file is part of Callgrind.
-   (c) 2003-2005, Josef Weidendorfer
+   This file is part of Callgrind, a Valgrind tool for call graph
+   profiling programs.
 
-   Parts are Copyright (C) 2002 Nicholas Nethercote
-      njn25@cam.ac.uk
+   Copyright (C) 2003-2005, Josef Weidendorfer (Josef.Weidendorfer@gmx.de)
 
+   This tool is derived from and contains code from Cachegrind
+   Copyright (C) 2002-2007 Nicholas Nethercote (njn@valgrind.org)
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -308,7 +309,7 @@ static CacheResult cachesim_ref(cache_t2* c, Addr a, UChar size)
 	return ((res1 == Miss) || (res2 == Miss)) ? Miss : Hit;
 
    } else {
-       VG_(printf)("addr: %x  size: %u  sets: %d %d", a, size, set1, set2);
+       VG_(printf)("addr: %lx  size: %u  sets: %d %d", a, size, set1, set2);
        VG_(tool_panic)("item straddles more than two cache sets");
    }
    return Hit;
@@ -415,7 +416,7 @@ CacheResult cachesim_ref_wb(cache_t2* c, RefType ref, Addr a, UChar size)
 	return ((res1 == Miss) || (res2 == Miss)) ? Miss : Hit;
 
    } else {
-       VG_(printf)("addr: %x  size: %u  sets: %d %d", a, size, set1, set2);
+       VG_(printf)("addr: %lx  size: %u  sets: %d %d", a, size, set1, set2);
        VG_(tool_panic)("item straddles more than two cache sets");
    }
    return Hit;
@@ -1642,7 +1643,7 @@ static void parse_opt ( cache_t* cache, char* orig_opt, int opt_len )
    return;
 
   bad:
-   VG_(bad_option)(orig_opt);
+   VG_(err_bad_option)(orig_opt);
 }
 
 /* Check for command line option for cache configuration.
