@@ -29,14 +29,17 @@
 */
 
 #include "pub_core_basics.h"
+#include "pub_core_vki.h"
+#include "pub_core_vkiscnums.h"
+#include "pub_core_debuglog.h"
 #include "pub_core_libcbase.h"
 #include "pub_core_libcassert.h"
 #include "pub_core_libcfile.h"
 #include "pub_core_libcprint.h"     // VG_(sprintf)
 #include "pub_core_libcproc.h"      // VG_(getpid), VG_(getppid)
+#include "pub_core_xarray.h"
 #include "pub_core_clientstate.h"   // VG_(fd_hard_limit)
 #include "pub_core_syscall.h"
-#include "vki_unistd.h"
 
 /* ---------------------------------------------------------------------
    File stuff
@@ -60,6 +63,7 @@ Int VG_(safe_fd)(Int oldfd)
    if (newfd != -1)
       VG_(close)(oldfd);
 
+   /* Set the close-on-exec flag for this fd. */
    VG_(fcntl)(newfd, VKI_F_SETFD, VKI_FD_CLOEXEC);
 
    vg_assert(newfd >= VG_(fd_hard_limit));
@@ -95,14 +99,30 @@ void VG_(close) ( Int fd )
 
 Int VG_(read) ( Int fd, void* buf, Int count)
 {
+   Int    ret;
    SysRes res = VG_(do_syscall3)(__NR_read, fd, (UWord)buf, count);
-   return res.isError ? -1 : res.val;
+   if (res.isError) {
+      ret = - (Int)(Word)res.err;
+      vg_assert(ret < 0);
+   } else {
+      ret = (Int)(Word)res.res;
+      vg_assert(ret >= 0);
+   }
+   return ret;
 }
 
 Int VG_(write) ( Int fd, const void* buf, Int count)
 {
+   Int    ret;
    SysRes res = VG_(do_syscall3)(__NR_write, fd, (UWord)buf, count);
-   return res.isError ? -1 : res.val;
+   if (res.isError) {
+      ret = - (Int)(Word)res.err;
+      vg_assert(ret < 0);
+   } else {
+      ret = (Int)(Word)res.res;
+      vg_assert(ret >= 0);
+   }
+   return ret;
 }
 
 Int VG_(pipe) ( Int fd[2] )
@@ -114,39 +134,61 @@ Int VG_(pipe) ( Int fd[2] )
 OffT VG_(lseek) ( Int fd, OffT offset, Int whence )
 {
    SysRes res = VG_(do_syscall3)(__NR_lseek, fd, offset, whence);
-   return res.isError ? (-1) : res.val;
+   return res.isError ? (-1) : res.res;
    /* if you change the error-reporting conventions of this, also
       change VG_(pread) and all other usage points. */
 }
 
 SysRes VG_(stat) ( Char* file_name, struct vki_stat* buf )
 {
+#  if defined(VGO_linux)
    SysRes res = VG_(do_syscall2)(__NR_stat, (UWord)file_name, (UWord)buf);
    return res;
+#  elif defined(VGO_aix5)
+   SysRes res = VG_(do_syscall4)(__NR_AIX5_statx,
+                                 (UWord)file_name,
+                                 (UWord)buf,
+                                 sizeof(struct vki_stat),
+                                 VKI_STX_NORMAL);
+   return res;
+#  else
+#    error Unknown OS
+#  endif
 }
 
 Int VG_(fstat) ( Int fd, struct vki_stat* buf )
 {
+#  if defined(VGO_linux)
    SysRes res = VG_(do_syscall2)(__NR_fstat, fd, (UWord)buf);
    return res.isError ? (-1) : 0;
+#  elif defined(VGO_aix5)
+   I_die_here;
+#  else
+#    error Unknown OS
+#  endif
 }
 
 Int VG_(fsize) ( Int fd )
 {
-#ifdef __NR_fstat64
+#  if defined(VGO_linux) && defined(__NR_fstat64)
    struct vki_stat64 buf;
    SysRes res = VG_(do_syscall2)(__NR_fstat64, fd, (UWord)&buf);
-#else
+   return res.isError ? (-1) : buf.st_size;
+#  elif defined(VGO_linux) && !defined(__NR_fstat64)
    struct vki_stat buf;
    SysRes res = VG_(do_syscall2)(__NR_fstat, fd, (UWord)&buf);
-#endif
    return res.isError ? (-1) : buf.st_size;
+#  elif defined(VGO_aix5)
+   I_die_here;
+#  else
+#  error Unknown OS
+#  endif
 }
 
 Bool VG_(is_dir) ( HChar* f )
 {
    struct vki_stat buf;
-   SysRes res = VG_(do_syscall2)(__NR_stat, (UWord)f, (UWord)&buf);
+   SysRes res = VG_(stat)(f, &buf);
    return res.isError ? False
                       : VKI_S_ISDIR(buf.st_mode) ? True : False;
 }
@@ -160,7 +202,7 @@ SysRes VG_(dup) ( Int oldfd )
 Int VG_(fcntl) ( Int fd, Int cmd, Int arg )
 {
    SysRes res = VG_(do_syscall3)(__NR_fcntl, fd, cmd, arg);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 }
 
 Int VG_(rename) ( Char* old_name, Char* new_name )
@@ -175,14 +217,71 @@ Int VG_(unlink) ( Char* file_name )
    return res.isError ? (-1) : 0;
 }
 
-/* Nb: we do not allow the Linux extension which malloc()s memory for the
-   buffer if buf==NULL, because we don't want Linux calling malloc() */
-Bool VG_(getcwd) ( Char* buf, SizeT size )
+/* The working directory at startup.  AIX doesn't provide an easy
+   system call to do getcwd, but fortunately we don't need arbitrary
+   getcwd support.  All that is really needed is to note the cwd at
+   process startup.  Hence VG_(record_startup_wd) notes it (in a
+   platform dependent way) and VG_(get_startup_wd) produces the noted
+   value.  Hence: */
+static HChar startup_wd[VKI_PATH_MAX];
+static Bool  startup_wd_acquired = False;
+
+/* Record the process' working directory at startup.  Is intended to
+   be called exactly once, at startup, before the working directory
+   changes.  Return True for success, False for failure, so that the
+   caller can bomb out suitably without creating module cycles if
+   there is a problem. */
+Bool VG_(record_startup_wd) ( void )
 {
-   SysRes res;
-   vg_assert(buf != NULL);
-   res = VG_(do_syscall2)(__NR_getcwd, (UWord)buf, size);
-   return res.isError ? False : True;
+   const Int szB = sizeof(startup_wd);
+   vg_assert(!startup_wd_acquired);
+   vg_assert(szB >= 512 && szB <= 16384/*let's say*/); /* stay sane */
+   VG_(memset)(startup_wd, 0, szB);
+#  if defined(VGO_linux)
+   /* Simple: just ask the kernel */
+   { SysRes res
+        = VG_(do_syscall2)(__NR_getcwd, (UWord)startup_wd, szB-1);
+     vg_assert(startup_wd[szB-1] == 0);
+     if (res.isError) {
+        return False;
+     } else {
+        startup_wd_acquired = True;
+        return True;
+     }
+   }
+#  elif defined(VGO_aix5)
+   /* We can't ask the kernel, so instead rely on launcher-aix5.c to
+      tell us the startup path.  Note the env var is keyed to the
+      parent's PID, not ours, since our parent is the launcher
+      process. */
+   { Char  envvar[100];
+     Char* wd = NULL;
+     VG_(memset)(envvar, 0, sizeof(envvar));
+     VG_(sprintf)(envvar, "VALGRIND_STARTUP_PWD_%d_XYZZY", 
+                          (Int)VG_(getppid)());
+     wd = VG_(getenv)( envvar );
+     if (wd == NULL || (1+VG_(strlen)(wd) >= szB))
+        return False;
+     VG_(strncpy_safely)(startup_wd, wd, szB);
+     vg_assert(startup_wd[szB-1] == 0);
+     startup_wd_acquired = True;
+     return True;
+   }
+#  else
+#    error Unknown OS
+#  endif
+}
+
+/* Copy the previously acquired startup_wd into buf[0 .. size-1],
+   or return False if buf isn't big enough. */
+Bool VG_(get_startup_wd) ( Char* buf, SizeT size )
+{
+   vg_assert(startup_wd_acquired);
+   vg_assert(startup_wd[ sizeof(startup_wd)-1 ] == 0);
+   if (1+VG_(strlen)(startup_wd) >= size)
+      return False;
+   VG_(strncpy_safely)(buf, startup_wd, size);
+   return True;
 }
 
 Int VG_(readlink) (Char* path, Char* buf, UInt bufsiz)
@@ -190,7 +289,7 @@ Int VG_(readlink) (Char* path, Char* buf, UInt bufsiz)
    SysRes res;
    /* res = readlink( path, buf, bufsiz ); */
    res = VG_(do_syscall3)(__NR_readlink, (UWord)path, (UWord)buf, bufsiz);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 }
 
 Int VG_(getdents) (UInt fd, struct vki_dirent *dirp, UInt count)
@@ -198,14 +297,14 @@ Int VG_(getdents) (UInt fd, struct vki_dirent *dirp, UInt count)
    SysRes res;
    /* res = getdents( fd, dirp, count ); */
    res = VG_(do_syscall3)(__NR_getdents, fd, (UWord)dirp, count);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 }
 
 /* Check accessibility of a file.  Returns zero for access granted,
    nonzero otherwise. */
 Int VG_(access) ( HChar* path, Bool irusr, Bool iwusr, Bool ixusr )
 {
-#if defined(VGO_linux)
+#  if defined(VGO_linux)
    /* Very annoyingly, I cannot find any definition for R_OK et al in
       the kernel interfaces.  Therefore I reluctantly resort to
       hardwiring in these magic numbers that I determined by
@@ -214,10 +313,16 @@ Int VG_(access) ( HChar* path, Bool irusr, Bool iwusr, Bool ixusr )
              | (iwusr ? 2/*W_OK*/ : 0)
              | (ixusr ? 1/*X_OK*/ : 0);
    SysRes res = VG_(do_syscall2)(__NR_access, (UWord)path, w);
-   return res.isError ? 1 : res.val;
-#else
-#  error "Don't know how to do VG_(access) on this OS"
-#endif
+   return res.isError ? 1 : 0;
+#  elif defined(VGO_aix5)
+   UWord w = (irusr ? VKI_R_OK : 0)
+             | (iwusr ? VKI_W_OK : 0)
+             | (ixusr ? VKI_X_OK : 0);
+   SysRes res = VG_(do_syscall2)(__NR_access, (UWord)path, w);
+   return res.isError ? 1 : 0;   
+#  else
+#    error "Don't know how to do VG_(access) on this OS"
+#  endif
 }
 
 /* 
@@ -227,27 +332,44 @@ Int VG_(access) ( HChar* path, Bool irusr, Bool iwusr, Bool ixusr )
    if group matches, then use the group permissions, else
    use other permissions.
 
-   Note that we can't deal with SUID/SGID, so we refuse to run them
-   (otherwise the executable may misbehave if it doesn't have the
-   permissions it thinks it does).
+   Note that we can't deal properly with SUID/SGID.  By default
+   (allow_setuid == False), we refuse to run them (otherwise the
+   executable may misbehave if it doesn't have the permissions it
+   thinks it does).  However, the caller may indicate that setuid
+   executables are allowed, for example if we are going to exec them
+   but not trace into them (iow, client sys_execve when
+   clo_trace_children == False).
+
+   If VKI_EACCES is returned (iow, permission was refused), then
+   *is_setuid is set to True iff permission was refused because the
+   executable is setuid.
 */
 /* returns: 0 = success, non-0 is failure */
-Int VG_(check_executable)(HChar* f)
+Int VG_(check_executable)(/*OUT*/Bool* is_setuid,
+                          HChar* f, Bool allow_setuid)
 {
-#ifdef __NR_stat64
+  /* This is something of a kludge.  Really we should fix VG_(stat) to
+     do this itself, but not clear how to do it as it depends on
+     having a 'struct vki_stat64' which is different from 'struct
+     vki_stat'. */
+#  if defined(VGO_linux) && defined(__NR_stat64)
    struct vki_stat64 st;
    SysRes res = VG_(do_syscall2)(__NR_stat64, (UWord)f, (UWord)&st);
-#else
+#  else
    struct vki_stat st;
-   SysRes res = VG_(do_syscall2)(__NR_stat, (UWord)f, (UWord)&st);
-#endif
+   SysRes res = VG_(stat)(f, &st);
+#  endif
+
+   if (is_setuid)
+      *is_setuid = False;
 
    if (res.isError) {
-      return res.val;
+      return res.err;
    }
 
-   if (st.st_mode & (VKI_S_ISUID | VKI_S_ISGID)) {
-      //VG_(printf)("Can't execute suid/sgid executable %s\n", exe);
+   if ( (st.st_mode & (VKI_S_ISUID | VKI_S_ISGID)) && !allow_setuid ) {
+      if (is_setuid)
+         *is_setuid = True;
       return VKI_EACCES;
    }
 
@@ -255,7 +377,7 @@ Int VG_(check_executable)(HChar* f)
       if (!(st.st_mode & VKI_S_IXUSR))
          return VKI_EACCES;
    } else {
-      int grpmatch = 0;
+      Int grpmatch = 0;
 
       if (VG_(getegid)() == st.st_gid)
 	 grpmatch = 1;
@@ -325,7 +447,7 @@ Int VG_(mkstemp) ( HChar* part_of_name, /*OUT*/HChar* fullname )
       if (sres.isError)
          continue;
       /* VG_(safe_fd) doesn't return if it fails. */
-      fd = VG_(safe_fd)( sres.val );
+      fd = VG_(safe_fd)( sres.res );
       if (fullname)
          VG_(strcpy)( fullname, buf );
       return fd;
@@ -345,7 +467,12 @@ static
 Int my_socket ( Int domain, Int type, Int protocol );
 
 static
-Int my_connect ( Int sockfd, struct vki_sockaddr_in* serv_addr, 
+Int my_connect ( Int sockfd, 
+#                if defined(VGO_linux)
+                   struct vki_sockaddr_in* serv_addr, 
+#                else
+                   void* serv_addr,
+#                endif
                  Int addrlen );
 
 UInt VG_(htonl) ( UInt x )
@@ -403,6 +530,9 @@ UShort VG_(ntohs) ( UShort x )
 */
 Int VG_(connect_via_socket)( UChar* str )
 {
+#if defined(VGO_aix5)
+   I_die_here;
+#else /* Yay, Linux */
    Int sd, res;
    struct vki_sockaddr_in servAddr;
    UInt   ip   = 0;
@@ -437,13 +567,14 @@ Int VG_(connect_via_socket)( UChar* str )
    }
 
    return sd;
+#endif
 }
 
 
 /* Let d = one or more digits.  Accept either:
    d.d.d.d  or  d.d.d.d:d
 */
-Int parse_inet_addr_and_port ( UChar* str, UInt* ip_addr, UShort* port )
+static Int parse_inet_addr_and_port ( UChar* str, UInt* ip_addr, UShort* port )
 {
 #  define GET_CH ((*str) ? (*str++) : 0)
    UInt ipa, i, j, c, any;
@@ -494,12 +625,15 @@ Int my_socket ( Int domain, Int type, Int protocol )
    args[1] = type;
    args[2] = protocol;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_SOCKET, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
    SysRes res;
    res = VG_(do_syscall3)(__NR_socket, domain, type, protocol );
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
 #  error Unknown arch
@@ -507,7 +641,12 @@ Int my_socket ( Int domain, Int type, Int protocol )
 }
 
 static
-Int my_connect ( Int sockfd, struct vki_sockaddr_in* serv_addr, 
+Int my_connect ( Int sockfd,
+#                if defined(VGO_linux)
+                   struct vki_sockaddr_in* serv_addr, 
+#                else
+                   void* serv_addr,
+#                endif
                  Int addrlen )
 {
 #if defined(VGP_x86_linux) || defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux)
@@ -517,12 +656,15 @@ Int my_connect ( Int sockfd, struct vki_sockaddr_in* serv_addr,
    args[1] = (UWord)serv_addr;
    args[2] = addrlen;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_CONNECT, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
    SysRes res;
    res = VG_(do_syscall3)(__NR_connect, sockfd, (UWord)serv_addr, addrlen);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
 #  error Unknown arch
@@ -532,10 +674,9 @@ Int my_connect ( Int sockfd, struct vki_sockaddr_in* serv_addr,
 Int VG_(write_socket)( Int sd, void *msg, Int count )
 {
    /* This is actually send(). */
-   /* Requests not to send SIGPIPE on errors on stream oriented
-      sockets when the other end breaks the connection. The EPIPE
-      error is still returned. */
-   Int flags = VKI_MSG_NOSIGNAL;
+   /* For Linux, VKI_MSG_NOSIGNAL is a request not to send SIGPIPE on
+      errors on stream oriented sockets when the other end breaks the
+      connection. The EPIPE error is still returned. */
 
 #if defined(VGP_x86_linux) || defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux)
    SysRes res;
@@ -543,14 +684,18 @@ Int VG_(write_socket)( Int sd, void *msg, Int count )
    args[0] = sd;
    args[1] = (UWord)msg;
    args[2] = count;
-   args[3] = flags;
+   args[3] = VKI_MSG_NOSIGNAL;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_SEND, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
    SysRes res;
-   res = VG_(do_syscall6)(__NR_sendto, sd, (UWord)msg, count, flags, 0,0);
-   return res.isError ? -1 : res.val;
+   res = VG_(do_syscall6)(__NR_sendto, sd, (UWord)msg, 
+                                       count, VKI_MSG_NOSIGNAL, 0,0);
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
 #  error Unknown arch
@@ -559,20 +704,23 @@ Int VG_(write_socket)( Int sd, void *msg, Int count )
 
 Int VG_(getsockname) ( Int sd, struct vki_sockaddr *name, Int *namelen)
 {
-   SysRes res;
-
 #if defined(VGP_x86_linux) || defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux)
+   SysRes res;
    UWord  args[3];
    args[0] = sd;
    args[1] = (UWord)name;
    args[2] = (UWord)namelen;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_GETSOCKNAME, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
+   SysRes res;
    res = VG_(do_syscall3)( __NR_getsockname,
                            (UWord)sd, (UWord)name, (UWord)namelen );
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
 #  error Unknown arch
@@ -581,32 +729,34 @@ Int VG_(getsockname) ( Int sd, struct vki_sockaddr *name, Int *namelen)
 
 Int VG_(getpeername) ( Int sd, struct vki_sockaddr *name, Int *namelen)
 {
-   SysRes res;
-
 #if defined(VGP_x86_linux) || defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux)
+   SysRes res;
    UWord  args[3];
    args[0] = sd;
    args[1] = (UWord)name;
    args[2] = (UWord)namelen;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_GETPEERNAME, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
+   SysRes res;
    res = VG_(do_syscall3)( __NR_getpeername,
                            (UWord)sd, (UWord)name, (UWord)namelen );
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
-#  error Unknown archx
+#  error Unknown arch
 #endif
 }
 
 Int VG_(getsockopt) ( Int sd, Int level, Int optname, void *optval,
                       Int *optlen)
 {
-   SysRes res;
-
 #if defined(VGP_x86_linux) || defined(VGP_ppc32_linux) || defined(VGP_ppc64_linux)
+   SysRes res;
    UWord  args[5];
    args[0] = sd;
    args[1] = level;
@@ -614,13 +764,17 @@ Int VG_(getsockopt) ( Int sd, Int level, Int optname, void *optval,
    args[3] = (UWord)optval;
    args[4] = (UWord)optlen;
    res = VG_(do_syscall2)(__NR_socketcall, VKI_SYS_GETSOCKOPT, (UWord)&args);
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
 
 #elif defined(VGP_amd64_linux)
+   SysRes res;
    res = VG_(do_syscall5)( __NR_getsockopt,
                            (UWord)sd, (UWord)level, (UWord)optname, 
                            (UWord)optval, (UWord)optlen );
-   return res.isError ? -1 : res.val;
+   return res.isError ? -1 : res.res;
+
+#elif defined(VGP_ppc32_aix5) || defined(VGP_ppc64_aix5)
+   I_die_here;
 
 #else
 #  error Unknown arch
