@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward
+   Copyright (C) 2000-2007 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -32,11 +32,33 @@
 #define __PUB_CORE_DEMANGLE_H
 
 //--------------------------------------------------------------------
-// PURPOSE: This module exports a single function for demangling C++
-// names.
+// PURPOSE: This module exports functions for demangling C++ and 
+// Z-encoded names.
 //--------------------------------------------------------------------
 
-extern void VG_(demangle) ( Char* orig, Char* result, Int result_size );
+/* This is the main, standard demangler entry point. */
+
+extern 
+void VG_(demangle) ( Bool do_cxx_demangle,
+                     Char* orig, Char* result, Int result_size );
+
+/* Demangle a Z-encoded name as described in pub_tool_redir.h. 
+   Z-encoded names are used by Valgrind for doing function 
+   interception/wrapping.
+
+   Demangle 'sym' into its soname and fnname parts, putting them in
+   the specified buffers.  Returns a Bool indicating whether the
+   demangled failed or not.  A failure can occur because the prefix
+   isn't recognised, the internal Z-escaping is wrong, or because one
+   or the other (or both) of the output buffers becomes full.  Passing
+   'so' as NULL is acceptable if the caller is only interested in the
+   function name part. */
+
+extern 
+Bool VG_(maybe_Z_demangle) ( const HChar* sym, 
+                             /*OUT*/HChar* so, Int soLen,
+                             /*OUT*/HChar* fn, Int fnLen,
+                             /*OUT*/Bool* isWrap );
 
 #endif   // __PUB_CORE_DEMANGLE_H
 
