@@ -32,7 +32,7 @@
 #define __PUB_TOOL_TOOLIFACE_H
 
 #include "pub_tool_errormgr.h"   // for Error, Supp
-#include "libvex.h"              // for VexGuestLayout
+#include "libvex.h"              // for all Vex stuff
 
 /* ------------------------------------------------------------------ */
 /* The interface version */
@@ -40,7 +40,7 @@
 /* The version number indicates binary-incompatible changes to the
    interface;  if the core and tool versions don't match, Valgrind
    will abort.  */
-#define VG_CORE_INTERFACE_VERSION   9
+#define VG_CORE_INTERFACE_VERSION   10
 
 typedef struct _ToolInfo {
    Int	sizeof_ToolInfo;
@@ -74,7 +74,7 @@ extern const ToolInfo VG_(tool_info);
 /* Basic tool functions */
 
 /* The tool_instrument function is passed as a callback to
-   LibVEX_Translate.  VgInstrumentClosure carries additional info
+   LibVEX_Translate.  VgCallbackClosure carries additional info
    which the instrumenter might like to know, but which is opaque to
    Vex.
 */
@@ -93,14 +93,151 @@ extern void VG_(basic_tool_funcs)(
 
    // Instrument a basic block.  Must be a true function, ie. the same
    // input always results in the same output, because basic blocks
-   // can be retranslated.  Unless you're doing something really
-   // strange...  Note that orig_addr_noredir is not necessarily the
-   // same as the address of the first instruction in the IR, due to
-   // function redirection.
-   IRBB*(*instrument)(VgCallbackClosure*, 
-                      IRBB* bb_in, 
-                      VexGuestLayout*, VexGuestExtents*, 
-                      IRType gWordTy, IRType hWordTy),
+   // can be retranslated, unless you're doing something really
+   // strange.  Anyway, the arguments.  Mostly they are straightforward
+   // except for the distinction between redirected and non-redirected
+   // guest code addresses, which is important to understand.
+   //
+   // VgCallBackClosure* closure contains extra arguments passed
+   // from Valgrind to the instrumenter, which Vex doesn't know about.
+   // You are free to look inside this structure.
+   //
+   // * closure->tid is the ThreadId of the thread requesting the
+   //   translation.  Not sure why this is here; perhaps callgrind
+   //   uses it.
+   //
+   // * closure->nraddr is the non-redirected guest address of the
+   //   start of the translation.  In other words, the translation is
+   //   being constructed because the guest program jumped to 
+   //   closure->nraddr but no translation of it was found.
+   //
+   // * closure->readdr is the redirected guest address, from which
+   //   the translation was really made.
+   //
+   //   To clarify this, consider what happens when, in Memcheck, the
+   //   first call to malloc() happens.  The guest program will be
+   //   trying to jump to malloc() in libc; hence ->nraddr will contain
+   //   that address.  However, Memcheck intercepts and replaces
+   //   malloc, hence ->readdr will be the address of Memcheck's
+   //   malloc replacement in
+   //   coregrind/m_replacemalloc/vg_replacemalloc.c.  It follows
+   //   that the first IMark in the translation will be labelled as
+   //   from ->readdr rather than ->nraddr.
+   //
+   //   Since most functions are not redirected, the majority of the
+   //   time ->nraddr will be the same as ->readdr.  However, you
+   //   cannot assume this: if your tool has metadata associated
+   //   with code addresses it will get into deep trouble if it does
+   //   make this assumption.
+   //
+   // IRSB* sb_in is the incoming superblock to be instrumented,
+   // in flat IR form.
+   //
+   // VexGuestLayout* layout contains limited info on the layout of
+   // the guest state: where the stack pointer and program counter
+   // are, and which fields should be regarded as 'always defined'.
+   // Memcheck uses this.
+   //
+   // VexGuestExtents* vge points to a structure which states the
+   // precise byte ranges of original code from which this translation
+   // was made (there may be up to three different ranges involved).
+   // Note again that these are the real addresses from which the code
+   // came.  And so it should be the case that closure->readdr is the
+   // same as vge->base[0]; indeed Cachegrind contains this assertion.
+   //
+   // Tools which associate shadow data with code addresses
+   // (cachegrind, callgrind) need to be particularly clear about
+   // whether they are making the association with redirected or
+   // non-redirected code addresses.  Both approaches are viable
+   // but you do need to understand what's going on.  See comments
+   // below on discard_basic_block_info().
+   //
+   // IRType gWordTy and IRType hWordTy contain the types of native
+   // words on the guest (simulated) and host (real) CPUs.  They will
+   // by either Ity_I32 or Ity_I64.  So far we have never built a
+   // cross-architecture Valgrind so they should always be the same.
+   //
+   /* --- Further comments about the IR that your --- */
+   /* --- instrumentation function will receive. --- */
+   /*
+      In the incoming IRSB, the IR for each instruction begins with an
+      IRStmt_IMark, which states the address and length of the
+      instruction from which this IR came.  This makes it easy for
+      profiling-style tools to know precisely which guest code
+      addresses are being executed.
+
+      However, before the first IRStmt_IMark, there may be other IR
+      statements -- a preamble.  In most cases this preamble is empty,
+      but when it isn't, what it contains is some supporting IR that
+      the JIT uses to ensure control flow works correctly.  This
+      preamble does not modify any architecturally defined guest state
+      (registers or memory) and so does not contain anything that will
+      be of interest to your tool.
+
+      You should therefore 
+
+      (1) copy any IR preceding the first IMark verbatim to the start
+          of the output IRSB.
+
+      (2) not try to instrument it or modify it in any way.
+
+      For the record, stuff that may be in the preamble at
+      present is:
+
+      - A self-modifying-code check has been requested for this block.
+        The preamble will contain instructions to checksum the block,
+        compare against the expected value, and exit the dispatcher
+        requesting a discard (hence forcing a retranslation) if they
+        don't match.
+
+      - This block is known to be the entry point of a wrapper of some
+        function F.  In this case the preamble contains code to write
+        the address of the original F (the fn being wrapped) into a
+        'hidden' guest state register _NRADDR.  The wrapper can later
+        read this register using a client request and make a
+        non-redirected call to it using another client-request-like
+        magic macro.
+
+      - For platforms that use the AIX ABI (including ppc64-linux), it
+        is necessary to have a preamble even for replacement functions
+        (not just for wrappers), because it is necessary to switch the
+        R2 register (constant-pool pointer) to a different value when
+        swizzling the program counter.
+
+        Hence the preamble pushes both R2 and LR (the return address)
+        on a small 16-entry stack in the guest state and sets R2 to an
+        appropriate value for the wrapper/replacement fn.  LR is then
+        set so that the wrapper/replacement fn returns to a magic IR
+        stub which restores R2 and LR and returns.
+
+        It's all hugely ugly and fragile.  And it places a stringent
+        requirement on m_debuginfo to find out the correct R2 (toc
+        pointer) value for the wrapper/replacement function.  So much
+        so that m_redir will refuse to honour a redirect-to-me request
+        if it cannot find (by asking m_debuginfo) a plausible R2 value
+        for 'me'.
+
+        Because this mechanism maintains a shadow stack of (R2,LR)
+        pairs in the guest state, it will fail if the
+        wrapper/redirection function, or anything it calls, longjumps
+        out past the wrapper, because then the magic return stub will
+        not be run and so the shadow stack will not be popped.  So it
+        will quickly fill up.  Fortunately none of this applies to
+        {x86,amd64,ppc32}-linux; on those platforms, wrappers can
+        longjump and recurse arbitrarily and everything should work
+        fine.
+
+      Note that copying the preamble verbatim may cause complications
+      for your instrumenter if you shadow IR temporaries.  See big
+      comment in MC_(instrument) in memcheck/mc_translate.c for
+      details.
+   */
+   IRSB*(*instrument)(VgCallbackClosure* closure, 
+                      IRSB*              sb_in, 
+                      VexGuestLayout*    layout, 
+                      VexGuestExtents*   vge, 
+                      IRType             gWordTy, 
+                      IRType             hWordTy),
 
    // Finish up, print out any results, etc.  `exitcode' is program's exit
    // code.  The shadow can be found with VG_(get_exit_status_shadow)().
@@ -112,7 +249,7 @@ extern void VG_(basic_tool_funcs)(
 
 /* Default value for avg_translations_sizeB (in bytes), indicating typical
    code expansion of about 6:1. */
-#define VG_DEFAULT_TRANS_SIZEB   100
+#define VG_DEFAULT_TRANS_SIZEB   172
 
 /* Information used in the startup message.  `name' also determines the
    string used for identifying suppressions in a suppression file as
@@ -166,6 +303,9 @@ extern void VG_(needs_tool_errors) (
    // Print error context.
    void (*pp_Error)(Error* err),
 
+   // Should the core indicate which ThreadId each error comes from?
+   Bool show_ThreadIDs_for_errors,
+
    // Should fill in any details that could be postponed until after the
    // decision whether to ignore the error (ie. details not affecting the
    // result of VG_(tdict).tool_eq_Error()).  This saves time when errors
@@ -208,29 +348,32 @@ extern void VG_(needs_tool_errors) (
    .so unloading, or otherwise at the discretion of m_transtab, eg
    when the table becomes too full) to avoid stale information being
    reused for new translations. */
-extern void VG_(needs_basic_block_discards) (
+extern void VG_(needs_superblock_discards) (
    // Discard any information that pertains to specific translations
    // or instructions within the address range given.  There are two
    // possible approaches.
    // - If info is being stored at a per-translation level, use orig_addr
    //   to identify which translation is being discarded.  Each translation
    //   will be discarded exactly once.
-   //   This orig_addr will match the orig_addr which was passed to
-   //   to instrument() when this translation was made.  Note that orig_addr
-   //   won't necessarily be the same as the first address in "extents".
+   //   This orig_addr will match the closure->nraddr which was passed to
+   //   to instrument() (see extensive comments above) when this 
+   //   translation was made.  Note that orig_addr won't necessarily be 
+   //   the same as the first address in "extents".
    // - If info is being stored at a per-instruction level, you can get
    //   the address range(s) being discarded by stepping through "extents".
    //   Note that any single instruction may belong to more than one
    //   translation, and so could be covered by the "extents" of more than
    //   one call to this function.
    // Doing it the first way (as eg. Cachegrind does) is probably easier.
-   void (*discard_basic_block_info)(Addr64 orig_addr, VexGuestExtents extents)
+   void (*discard_superblock_info)(Addr64 orig_addr, VexGuestExtents extents)
 );
 
 /* Tool defines its own command line options? */
 extern void VG_(needs_command_line_options) (
    // Return True if option was recognised.  Presumably sets some state to
-   // record the option as well.
+   // record the option as well.  Nb: tools can assume that the argv will
+   // never disappear.  So they can, for example, store a pointer to a string
+   // within an option, rather than having to make a copy.
    Bool (*process_cmd_line_option)(Char* argv),
 
    // Print out command line usage for options for normal tool operation.
@@ -276,9 +419,6 @@ extern void VG_(needs_sanity_checks) (
 /* Do we need to see data symbols? */
 extern void VG_(needs_data_syms) ( void );
 
-/* Does the tool need shadow memory allocated? */
-extern void VG_(needs_shadow_memory)( void );
-
 /* Does the tool replace malloc() and friends with its own versions?
    This has to be combined with the use of a vgpreload_<tool>.so module
    or it won't work.  See massif/Makefile.am for how to build it. */
@@ -301,13 +441,19 @@ extern void VG_(needs_malloc_replacement)(
  * it". */
 extern void VG_(needs_xml_output)( void );
 
+/* Does the tool want to have one final pass over the IR after tree
+   building but before instruction selection?  If so specify the
+   function here. */
+extern void VG_(needs_final_IR_tidy_pass) ( IRSB*(*final_tidy)(IRSB*) );
+
+
 /* ------------------------------------------------------------------ */
 /* Core events to track */
 
 /* Part of the core from which this call was made.  Useful for determining
    what kind of error message should be emitted. */
 typedef
-   enum { Vg_CoreStartup, Vg_CorePThread, Vg_CoreSignal, Vg_CoreSysCall,
+   enum { Vg_CoreStartup, Vg_CoreSignal, Vg_CoreSysCall,
           Vg_CoreTranslate, Vg_CoreClientReq }
    CorePart;
 
@@ -399,31 +545,65 @@ void VG_(track_post_reg_write_clientcall_return)(
 
 
 /* Scheduler events (not exhaustive) */
-void VG_(track_thread_run)(void(*f)(ThreadId tid));
+
+/* Called when 'tid' starts or stops running client code blocks.
+   Gives the total dispatched block count at that event.  Note, this
+   is not the same as 'tid' holding the BigLock (the lock that ensures
+   that only one thread runs at a time): a thread can hold the lock
+   for other purposes (making translations, etc) yet not be running
+   client blocks.  Obviously though, a thread must hold the lock in
+   order to run client code blocks, so the times bracketed by
+   'start_client_code'..'stop_client_code' are a subset of the times
+   when thread 'tid' holds the cpu lock.
+*/
+void VG_(track_start_client_code)(
+        void(*f)(ThreadId tid, ULong blocks_dispatched)
+     );
+void VG_(track_stop_client_code)(
+        void(*f)(ThreadId tid, ULong blocks_dispatched)
+     );
 
 
 /* Thread events (not exhaustive)
 
-   Called during thread create, before the new thread has run any
-   instructions (or touched any memory).
- */
-void VG_(track_post_thread_create)(void(*f)(ThreadId tid, ThreadId child));
-void VG_(track_post_thread_join)  (void(*f)(ThreadId joiner, ThreadId joinee));
+   ll_create: low level thread creation.  Called before the new thread
+   has run any instructions (or touched any memory).  In fact, called
+   immediately before the new thread has come into existence; the new
+   thread can be assumed to exist when notified by this call.
 
-/* Mutex events (not exhaustive)
-   "void *mutex" is really a pthread_mutex *
+   ll_exit: low level thread exit.  Called after the exiting thread
+   has run its last instruction.
 
-   Called before a thread can block while waiting for a mutex (called
-   regardless of whether the thread will block or not).  */
-void VG_(track_pre_mutex_lock)(void(*f)(ThreadId tid, void* mutex));
+   The _ll_ part makes it clear these events are not to do with
+   pthread_create or pthread_exit/pthread_join (etc), which are a
+   higher level abstraction synthesised by libpthread.  What you can
+   be sure of from _ll_create/_ll_exit is the absolute limits of each
+   thread's lifetime, and hence be assured that all memory references
+   made by the thread fall inside the _ll_create/_ll_exit pair.  This
+   is important for tools that need a 100% accurate account of which
+   thread is responsible for every memory reference in the process.
 
-/* Called once the thread actually holds the mutex (always paired with
-   pre_mutex_lock).  */
-void VG_(track_post_mutex_lock)(void(*f)(ThreadId tid, void* mutex));
+   pthread_create/join/exit do not give this property.  Calls/returns
+   to/from them happen arbitrarily far away from the relevant
+   low-level thread create/quit event.  In general a few hundred
+   instructions; hence a few hundred(ish) memory references could get
+   misclassified each time.
+
+   pre_thread_first_insn: is called when the thread is all set up and
+   ready to go (stack in place, etc) but has not executed its first
+   instruction yet.  Gives threading tools a chance to ask questions
+   about the thread (eg, what is its initial client stack pointer)
+   that are not easily answered at pre_thread_ll_create time.
+
+   For a given thread, the call sequence is:
+      ll_create (in the parent's context)
+      first_insn (in the child's context)
+      ll_exit (in the child's context)
+*/
+void VG_(track_pre_thread_ll_create) (void(*f)(ThreadId tid, ThreadId child));
+void VG_(track_pre_thread_first_insn)(void(*f)(ThreadId tid));
+void VG_(track_pre_thread_ll_exit)   (void(*f)(ThreadId tid));
 
-/* Called after a thread has released a mutex (no need for a corresponding
-   pre_mutex_unlock, because unlocking can't block).  */
-void VG_(track_post_mutex_unlock)(void(*f)(ThreadId tid, void* mutex));
 
 /* Signal events (not exhaustive)
 
@@ -437,10 +617,6 @@ void VG_(track_pre_deliver_signal) (void(*f)(ThreadId tid, Int sigNo,
    handler longjmps, this won't be called.  */
 void VG_(track_post_deliver_signal)(void(*f)(ThreadId tid, Int sigNo));
 
-/* Others... condition variables...
-   ...
- */
-
 #endif   // __PUB_TOOL_TOOLIFACE_H
 
 /*--------------------------------------------------------------------*/
