@@ -7,7 +7,7 @@
    This file is part of MemCheck, a heavyweight Valgrind tool for
    detecting memory errors.
 
-   Copyright (C) 2000-2006 Julian Seward 
+   Copyright (C) 2000-2007 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -29,6 +29,7 @@
 */
 
 #include "pub_tool_basics.h"
+#include "pub_tool_vki.h"
 #include "pub_tool_aspacemgr.h"
 #include "pub_tool_execontext.h"
 #include "pub_tool_hashtable.h"
@@ -108,21 +109,6 @@ static Addr* get_seg_starts ( /*OUT*/Int* n_acquired )
 /*--- Detecting leaked (unreachable) malloc'd blocks.      ---*/
 /*------------------------------------------------------------*/
 
-/* A block is either 
-   -- Proper-ly reached; a pointer to its start has been found
-   -- Interior-ly reached; only an interior pointer to it has been found
-   -- Unreached; so far, no pointers to any part of it have been found. 
-   -- IndirectLeak; leaked, but referred to by another leaked block
-*/
-typedef 
-   enum { 
-      Unreached    =0, 
-      IndirectLeak =1,
-      Interior     =2, 
-      Proper       =3
-  }
-  Reachedness;
-
 /* An entry in the mark stack */
 typedef 
    struct {
@@ -132,30 +118,6 @@ typedef
    } 
    MarkStack;
 
-/* A block record, used for generating err msgs. */
-typedef
-   struct _LossRecord {
-      struct _LossRecord* next;
-      /* Where these lost blocks were allocated. */
-      ExeContext*  allocated_at;
-      /* Their reachability. */
-      Reachedness  loss_mode;
-      /* Number of blocks and total # bytes involved. */
-      SizeT        total_bytes;
-      SizeT        indirect_bytes;
-      UInt         num_blocks;
-   }
-   LossRecord;
-
-/* The 'extra' struct for leak errors. */
-typedef 
-   struct {
-      UInt        n_this_record;
-      UInt        n_total_records;
-      LossRecord* lossRecord;
-   }
-   LeakExtra;
-
 /* Find the i such that ptr points at or inside the block described by
    shadows[i].  Return -1 if none found.  This assumes that shadows[]
    has been sorted on the ->data field. */
@@ -174,8 +136,8 @@ Int find_shadow_for_OLD ( Addr       ptr,
    for (i = 0; i < n_shadows; i++) {
       PROF_EVENT(71, "find_shadow_for_OLD(loop)");
       a_lo = shadows[i]->data;
-      a_hi = ((Addr)shadows[i]->data) + shadows[i]->size;
-      if (a_lo <= ptr && ptr <= a_hi)
+      a_hi = ((Addr)shadows[i]->data) + shadows[i]->szB;
+      if (a_lo <= ptr && ptr < a_hi)
          return i;
    }
    return -1;
@@ -200,17 +162,25 @@ Int find_shadow_for ( Addr       ptr,
 
       mid      = (lo + hi) / 2;
       a_mid_lo = shadows[mid]->data;
-      a_mid_hi = shadows[mid]->data + shadows[mid]->size;
+      a_mid_hi = shadows[mid]->data + shadows[mid]->szB;
+      /* Extent of block 'mid' is [a_mid_lo .. a_mid_hi).
+         Special-case zero-sized blocks - treat them as if they had
+         size 1.  Not doing so causes them to not cover any address
+         range at all and so will never be identified as the target of
+         any pointer, which causes them to be incorrectly reported as
+         definitely leaked. */
+      if (shadows[mid]->szB == 0)
+         a_mid_hi++;
 
       if (ptr < a_mid_lo) {
          hi = mid-1;
          continue;
       } 
-      if (ptr > a_mid_hi) {
+      if (ptr >= a_mid_hi) {
          lo = mid+1;
          continue;
       }
-      tl_assert(ptr >= a_mid_lo && ptr <= a_mid_hi);
+      tl_assert(ptr >= a_mid_lo && ptr < a_mid_hi);
       retVal = mid;
       break;
    }
@@ -234,83 +204,6 @@ static SizeT	  lc_scanned;
 static Bool	  (*lc_is_within_valid_secondary) (Addr addr);
 static Bool	  (*lc_is_valid_aligned_word)     (Addr addr);
 
-static const HChar* str_lossmode ( Reachedness lossmode )
-{
-   const HChar *loss = "?";
-   switch (lossmode) {
-      case Unreached:    loss = "definitely lost"; break;
-      case IndirectLeak: loss = "indirectly lost"; break;
-      case Interior:     loss = "possibly lost"; break;
-      case Proper:       loss = "still reachable"; break;
-   }
-   return loss;
-}
-
-static const HChar* xml_kind ( Reachedness lossmode )
-{
-   const HChar *loss = "?";
-   switch (lossmode) {
-      case Unreached:    loss = "Leak_DefinitelyLost"; break;
-      case IndirectLeak: loss = "Leak_IndirectlyLost"; break;
-      case Interior:     loss = "Leak_PossiblyLost"; break;
-      case Proper:       loss = "Leak_StillReachable"; break;
-   }
-   return loss;
-}
-
-
-/* Used for printing leak errors, avoids exposing the LossRecord type (which
-   comes in as void*, requiring a cast. */
-void MC_(pp_LeakError)(void* vextra)
-{
-   HChar* xpre  = VG_(clo_xml) ? "  <what>" : "";
-   HChar* xpost = VG_(clo_xml) ? "</what>"  : "";
-
-   LeakExtra* extra = (LeakExtra*)vextra;
-   LossRecord* l    = extra->lossRecord;
-   const Char *loss = str_lossmode(l->loss_mode);
-
-   if (VG_(clo_xml)) {
-      VG_(message)(Vg_UserMsg, "  <kind>%t</kind>", xml_kind(l->loss_mode));
-   } else {
-      VG_(message)(Vg_UserMsg, "");
-   }
-
-   if (l->indirect_bytes) {
-      VG_(message)(Vg_UserMsg, 
-         "%s%,lu (%,lu direct, %,lu indirect) bytes in %,u blocks"
-         " are %s in loss record %,u of %,u%s",
-         xpre,
-         l->total_bytes + l->indirect_bytes, 
-         l->total_bytes, l->indirect_bytes, l->num_blocks,
-         loss, extra->n_this_record, extra->n_total_records,
-         xpost
-      );
-      if (VG_(clo_xml)) {
-         // Nb: don't put commas in these XML numbers 
-         VG_(message)(Vg_UserMsg, "  <leakedbytes>%lu</leakedbytes>", 
-                                  l->total_bytes + l->indirect_bytes);
-         VG_(message)(Vg_UserMsg, "  <leakedblocks>%u</leakedblocks>", 
-                                  l->num_blocks);
-      }
-   } else {
-      VG_(message)(
-         Vg_UserMsg, 
-         "%s%,lu bytes in %,u blocks are %s in loss record %,u of %,u%s",
-         xpre,
-         l->total_bytes, l->num_blocks,
-         loss, extra->n_this_record, extra->n_total_records,
-         xpost
-      );
-      if (VG_(clo_xml)) {
-         VG_(message)(Vg_UserMsg, "  <leakedbytes>%d</leakedbytes>", 
-                                  l->total_bytes);
-         VG_(message)(Vg_UserMsg, "  <leakedblocks>%d</leakedblocks>", 
-                                  l->num_blocks);
-      }
-   }
-   VG_(pp_ExeContext)(l->allocated_at);
-}
 
 SizeT MC_(bytes_leaked)     = 0;
 SizeT MC_(bytes_indirect)   = 0;
@@ -345,12 +238,15 @@ static void lc_markstack_push_WRK(Addr ptr, Int clique)
       return;
 
    tl_assert(sh_no >= 0 && sh_no < lc_n_shadows);
-   tl_assert(ptr <= lc_shadows[sh_no]->data + lc_shadows[sh_no]->size);
+   tl_assert(ptr >= lc_shadows[sh_no]->data);
+   tl_assert(ptr < lc_shadows[sh_no]->data 
+                   + lc_shadows[sh_no]->szB
+                   + (lc_shadows[sh_no]->szB==0  ? 1  : 0));
 
    if (lc_markstack[sh_no].state == Unreached) {
       if (0)
 	 VG_(printf)("pushing %p-%p\n", lc_shadows[sh_no]->data, 
-		     lc_shadows[sh_no]->data + lc_shadows[sh_no]->size);
+		     lc_shadows[sh_no]->data + lc_shadows[sh_no]->szB);
 
       tl_assert(lc_markstack[sh_no].next == -1);
       lc_markstack[sh_no].next = lc_markstack_top;
@@ -377,15 +273,15 @@ static void lc_markstack_push_WRK(Addr ptr, Int clique)
 	 if (sh_no != clique) {
 	    if (VG_DEBUG_CLIQUE) {
 	       if (lc_markstack[sh_no].indirect)
-		  VG_(printf)("  clique %d joining clique %d adding %d+%d bytes\n", 
+		  VG_(printf)("  clique %d joining clique %d adding %lu+%lu bytes\n", 
 			      sh_no, clique, 
-			      lc_shadows[sh_no]->size, lc_markstack[sh_no].indirect);
+			      lc_shadows[sh_no]->szB, lc_markstack[sh_no].indirect);
 	       else
-		  VG_(printf)("  %d joining %d adding %d\n", 
-			      sh_no, clique, lc_shadows[sh_no]->size);
+		  VG_(printf)("  %d joining %d adding %lu\n", 
+			      sh_no, clique, lc_shadows[sh_no]->szB);
 	    }
 
-	    lc_markstack[clique].indirect += lc_shadows[sh_no]->size;
+	    lc_markstack[clique].indirect += lc_shadows[sh_no]->szB;
 	    lc_markstack[clique].indirect += lc_markstack[sh_no].indirect;
 	    lc_markstack[sh_no].indirect = 0; /* shouldn't matter */
 	 }
@@ -491,7 +387,7 @@ static void lc_do_leakcheck(Int clique)
       tl_assert(top >= 0 && top < lc_n_shadows);      
       tl_assert(lc_markstack[top].state != Unreached);
 
-      lc_scan_memory_WRK(lc_shadows[top]->data, lc_shadows[top]->size, clique);
+      lc_scan_memory_WRK(lc_shadows[top]->data, lc_shadows[top]->szB, clique);
    }
 }
 
@@ -508,7 +404,6 @@ static void full_report(ThreadId tid)
    LossRecord* errlist;
    LossRecord* p;
    Bool   is_suppressed;
-   LeakExtra leak_extra;
 
    /* Go through and group lost structures into cliques.  For each
       Unreached block, push it onto the mark stack, and find all the
@@ -518,8 +413,8 @@ static void full_report(ThreadId tid)
       pass), then the cliques are merged. */
    for (i = 0; i < lc_n_shadows; i++) {
       if (VG_DEBUG_CLIQUE)
-	 VG_(printf)("cliques: %d at %p -> %s\n",
-		     i, lc_shadows[i]->data, str_lossmode(lc_markstack[i].state));
+	 VG_(printf)("cliques: %d at %p -> Loss state %d\n",
+		     i, lc_shadows[i]->data, lc_markstack[i].state);
       if (lc_markstack[i].state != Unreached)
 	 continue;
 
@@ -566,14 +461,14 @@ static void full_report(ThreadId tid)
       }
       if (p != NULL) {
          p->num_blocks  ++;
-         p->total_bytes += lc_shadows[i]->size;
+         p->total_bytes += lc_shadows[i]->szB;
 	 p->indirect_bytes += lc_markstack[i].indirect;
       } else {
          n_lossrecords ++;
          p = VG_(malloc)(sizeof(LossRecord));
          p->loss_mode    = lc_markstack[i].state;
          p->allocated_at = where;
-         p->total_bytes  = lc_shadows[i]->size;
+         p->total_bytes  = lc_shadows[i]->szB;
 	 p->indirect_bytes = lc_markstack[i].indirect;
          p->num_blocks   = 1;
          p->next         = errlist;
@@ -607,11 +502,8 @@ static void full_report(ThreadId tid)
       // Nb: because VG_(unique_error) does all the error processing
       // immediately, and doesn't save the error, leakExtra can be
       // stack-allocated.
-      leak_extra.n_this_record   = i+1;
-      leak_extra.n_total_records = n_lossrecords;
-      leak_extra.lossRecord      = p_min;
       is_suppressed = 
-         MC_(record_leak_error) ( tid, &leak_extra, p_min->allocated_at,
+         MC_(record_leak_error) ( tid, i+1, n_lossrecords, p_min,
                                   print_record );
 
       if (is_suppressed) {
@@ -647,7 +539,7 @@ static void make_summary(void)
    Int i;
 
    for(i = 0; i < lc_n_shadows; i++) {
-      SizeT size = lc_shadows[i]->size;
+      SizeT size = lc_shadows[i]->szB;
 
       switch(lc_markstack[i].state) {
       case Unreached:
@@ -673,6 +565,107 @@ static void make_summary(void)
    }
 }
 
+static MC_Chunk**
+find_active_shadows(UInt* n_shadows)
+{
+   /* Our goal is to construct a set of shadows that includes every
+    * mempool chunk, and every malloc region that *doesn't* contain a
+    * mempool chunk. We do this in several phases.
+    *
+    * First we collect all the malloc chunks into an array and sort it.
+    * We do this because we want to query the chunks by interior
+    * pointers, requiring binary search.
+    *
+    * Second we build an array containing a Bool for each malloc chunk,
+    * indicating whether it contains any mempools.
+    *
+    * Third we loop over the mempool tables. For each chunk in each
+    * pool, we set the entry in the Bool array corresponding to the
+    * malloc chunk containing the mempool chunk.
+    *
+    * Finally we copy the mempool chunks and the non-marked malloc
+    * chunks into a combined array of shadows, free our temporaries,
+    * and return the combined array.
+    */
+
+   MC_Mempool *mp;
+   MC_Chunk **mallocs, **shadows, *mc;
+   UInt n_mallocs, m, s;
+   Bool *malloc_chunk_holds_a_pool_chunk;
+
+   mallocs = (MC_Chunk**) VG_(HT_to_array)( MC_(malloc_list), &n_mallocs );
+
+   if (n_mallocs == 0) {
+      tl_assert(mallocs == NULL);
+      *n_shadows = 0;
+      return NULL;
+   }
+
+   VG_(ssort)((void*)mallocs, n_mallocs, 
+              sizeof(VgHashNode*), lc_compar);
+
+   malloc_chunk_holds_a_pool_chunk = VG_(calloc)( n_mallocs, sizeof(Bool) );
+
+   *n_shadows = n_mallocs;
+
+   VG_(HT_ResetIter)(MC_(mempool_list));
+   while ( (mp = VG_(HT_Next)(MC_(mempool_list))) ) {
+      VG_(HT_ResetIter)(mp->chunks);
+      while ( (mc = VG_(HT_Next)(mp->chunks)) ) {
+
+         /* We'll need a shadow for this chunk. */
+         ++(*n_shadows);
+
+         /* Possibly invalidate the malloc holding the beginning of this chunk. */
+         m = find_shadow_for(mc->data, mallocs, n_mallocs);
+         if (m != -1 && malloc_chunk_holds_a_pool_chunk[m] == False) {
+            tl_assert(*n_shadows > 0);
+            --(*n_shadows);
+            malloc_chunk_holds_a_pool_chunk[m] = True;
+         }
+
+         /* Possibly invalidate the malloc holding the end of this chunk. */
+         if (mc->szB > 1) {
+            m = find_shadow_for(mc->data + (mc->szB - 1), mallocs, n_mallocs);
+            if (m != -1 && malloc_chunk_holds_a_pool_chunk[m] == False) {
+               tl_assert(*n_shadows > 0);
+               --(*n_shadows);
+               malloc_chunk_holds_a_pool_chunk[m] = True;
+            }
+         }
+      }
+   }
+
+   tl_assert(*n_shadows > 0);
+   shadows = VG_(malloc)(sizeof(VgHashNode*) * (*n_shadows));
+   s = 0;
+
+   /* Copy the mempool chunks into the final array. */
+   VG_(HT_ResetIter)(MC_(mempool_list));
+   while ( (mp = VG_(HT_Next)(MC_(mempool_list))) ) {
+      VG_(HT_ResetIter)(mp->chunks);
+      while ( (mc = VG_(HT_Next)(mp->chunks)) ) {
+         tl_assert(s < *n_shadows);
+         shadows[s++] = mc;
+      }
+   }
+
+   /* Copy the malloc chunks into the final array. */
+   for (m = 0; m < n_mallocs; ++m) {
+      if (!malloc_chunk_holds_a_pool_chunk[m]) {
+         tl_assert(s < *n_shadows);
+         shadows[s++] = mallocs[m];
+      }
+   }
+
+   tl_assert(s == *n_shadows);
+   VG_(free)(mallocs);
+   VG_(free)(malloc_chunk_holds_a_pool_chunk);
+
+   return shadows;
+}
+
+
 /* Top level entry point to leak detector.  Call here, passing in
    suitable address-validating functions (see comment at top of
    scan_all_valid_memory above).  These functions used to encapsulate the
@@ -689,9 +682,7 @@ void MC_(do_detect_memory_leaks) (
    
    tl_assert(mode != LC_Off);
 
-   /* VG_(HT_to_array) allocates storage for shadows */
-   lc_shadows = (MC_Chunk**)VG_(HT_to_array)( MC_(malloc_list),
-                                              &lc_n_shadows );
+   lc_shadows = find_active_shadows(&lc_n_shadows);
 
    /* Sort the array. */
    VG_(ssort)((void*)lc_shadows, lc_n_shadows, sizeof(VgHashNode*), lc_compar);
@@ -701,10 +692,22 @@ void MC_(do_detect_memory_leaks) (
       tl_assert( lc_shadows[i]->data <= lc_shadows[i+1]->data);
    }
 
-   /* Sanity check -- make sure they don't overlap */
+   /* Sanity check -- make sure they don't overlap.  But do allow
+      exact duplicates.  If this assertion fails, it may mean that the
+      application has done something stupid with
+      VALGRIND_MALLOCLIKE_BLOCK client requests, specifically, has
+      made overlapping requests (which are nonsensical).  Another way
+      to screw up is to use VALGRIND_MALLOCLIKE_BLOCK for stack
+      locations; again nonsensical. */
    for (i = 0; i < lc_n_shadows-1; i++) {
-      tl_assert( lc_shadows[i]->data + lc_shadows[i]->size
-                 <= lc_shadows[i+1]->data );
+      tl_assert( /* normal case - no overlap */
+                 (lc_shadows[i]->data + lc_shadows[i]->szB
+                  <= lc_shadows[i+1]->data )
+                 ||
+                 /* degenerate case: exact duplicates */
+                 (lc_shadows[i]->data == lc_shadows[i+1]->data
+                  && lc_shadows[i]->szB == lc_shadows[i+1]->szB)
+               );
    }
 
    if (lc_n_shadows == 0) {
@@ -723,7 +726,7 @@ void MC_(do_detect_memory_leaks) (
 
    lc_min_mallocd_addr = lc_shadows[0]->data;
    lc_max_mallocd_addr = lc_shadows[lc_n_shadows-1]->data
-                         + lc_shadows[lc_n_shadows-1]->size;
+                         + lc_shadows[lc_n_shadows-1]->szB;
 
    lc_markstack = VG_(malloc)( lc_n_shadows * sizeof(*lc_markstack) );
    for (i = 0; i < lc_n_shadows; i++) {
@@ -748,14 +751,13 @@ void MC_(do_detect_memory_leaks) (
       categorisation, which [if the users ever manage to understand it]
       is really useful for detecting lost cycles.
    */
-   { NSegment* seg;
-     Addr*     seg_starts;
+   { Addr*     seg_starts;
      Int       n_seg_starts;
      seg_starts = get_seg_starts( &n_seg_starts );
      tl_assert(seg_starts && n_seg_starts > 0);
      /* VG_(am_show_nsegments)( 0,"leakcheck"); */
      for (i = 0; i < n_seg_starts; i++) {
-        seg = VG_(am_find_nsegment)( seg_starts[i] );
+        NSegment const* seg = VG_(am_find_nsegment)( seg_starts[i] );
         tl_assert(seg);
         if (seg->kind != SkFileC && seg->kind != SkAnonC) 
            continue;
@@ -763,6 +765,21 @@ void MC_(do_detect_memory_leaks) (
            continue;
         if (seg->isCH)
            continue;
+
+        /* Don't poke around in device segments as this may cause
+           hangs.  Exclude /dev/zero just in case someone allocated
+           memory by explicitly mapping /dev/zero. */
+        if (seg->kind == SkFileC 
+            && (VKI_S_ISCHR(seg->mode) || VKI_S_ISBLK(seg->mode))) {
+           HChar* dev_name = VG_(am_get_filename)( (NSegment*)seg );
+           if (dev_name && 0 == VG_(strcmp)(dev_name, "/dev/zero")) {
+              /* don't skip /dev/zero */
+           } else {
+              /* skip this device mapping */
+              continue;
+           }
+        }
+
         if (0)
            VG_(printf)("ACCEPT %2d  %p %p\n", i, seg->start, seg->end);
         lc_scan_memory(seg->start, seg->end+1 - seg->start);
@@ -803,14 +820,17 @@ void MC_(do_detect_memory_leaks) (
                                MC_(bytes_reachable), blocks_reachable );
       VG_(message)(Vg_UserMsg, "        suppressed: %,lu bytes in %,lu blocks.",
                                MC_(bytes_suppressed), blocks_suppressed );
-      if (mode == LC_Summary && blocks_leaked > 0)
-	 VG_(message)(Vg_UserMsg,
-		      "Use --leak-check=full to see details of leaked memory.");
-      else if (!MC_(clo_show_reachable)) {
+      if (mode == LC_Summary 
+          && (blocks_leaked + blocks_indirect 
+              + blocks_dubious + blocks_reachable) > 0) {
+         VG_(message)(Vg_UserMsg,
+                      "Rerun with --leak-check=full to see details of leaked memory.");
+      }
+      if (blocks_reachable > 0 && !MC_(clo_show_reachable) && mode == LC_Full) {
          VG_(message)(Vg_UserMsg, 
            "Reachable blocks (those to which a pointer was found) are not shown.");
          VG_(message)(Vg_UserMsg, 
-            "To see them, rerun with: --show-reachable=yes");
+            "To see them, rerun with: --leak-check=full --show-reachable=yes");
       }
    }
 
