@@ -1,7 +1,7 @@
 
 int main ( void )
 {
-#if defined(__powerpc64__)
+#if defined(__powerpc64__) || defined(_AIX)
    /* on ppc64-linux, a function pointer points to a function
       descriptor, not to the function's entry point.  Hence to get
       uniform behaviour on all supported targets - a jump to 0xE000000
