@@ -178,7 +178,8 @@ test_strcpy (void)
   { const void *src = "frobozz";
     void *dst = one;
     check (strcpy (dst, src) == dst, 1);
-    equal (dst, "frobozz", 2); }
+    equal (dst, "frobozz", 2);
+  }
 }
 
 static void
@@ -462,6 +463,7 @@ test_strchr (void)
    }
 }
 
+#if !defined(_AIX)
 static void
 test_strchrnul (void)
 {
@@ -495,7 +497,9 @@ test_strchrnul (void)
       }
    }
 }
+#endif /* !defined(_AIX) */
 
+#if !defined(_AIX)
 static void
 test_rawmemchr (void)
 {
@@ -522,6 +526,7 @@ test_rawmemchr (void)
       }
    }
 }
+#endif /* !defined(_AIX) */
 
 static void
 test_index (void)
@@ -569,6 +574,7 @@ test_strrchr (void)
    }
 }
 
+#if !defined(_AIX)
 static void
 test_memrchr (void)
 {
@@ -614,6 +620,7 @@ test_memrchr (void)
     }
   }
 }
+#endif /* !defined(_AIX) */
 
 static void
 test_rindex (void)
@@ -889,6 +896,7 @@ test_strsep (void)
   equal(one+4, "c", 50);
 
   {
+#   if !defined(_AIX)
     char text[] = "This,is,a,test";
     char *list = strdupa (text);
     equal (strsep (&list, ","), "This", 51);
@@ -896,6 +904,7 @@ test_strsep (void)
     equal (strsep (&list, ","), "a", 53);
     equal (strsep (&list, ","), "test", 54);
     check (strsep (&list, ",") == NULL, 55);
+#   endif
   }
 
   cp = strcpy(one, "a,b, c,, ,d,");
@@ -1048,6 +1057,7 @@ test_memcpy (void)
     }
 }
 
+#if !defined(_AIX)
 static void
 test_mempcpy (void)
 {
@@ -1084,6 +1094,7 @@ test_mempcpy (void)
       equal (two, "hi there", 12 + (i * 6));
     }
 }
+#endif /* !defined(_AIX) */
 
 static void
 test_memmove (void)
@@ -1389,11 +1400,15 @@ main (void)
   /* strchr.  */
   test_strchr ();
 
+# if !defined(_AIX)
   /* strchrnul.  */
   test_strchrnul ();
+# endif
 
+# if !defined(_AIX)
   /* rawmemchr.  */
   test_rawmemchr ();
+# endif
 
   /* index - just like strchr.  */
   test_index ();
@@ -1401,8 +1416,10 @@ main (void)
   /* strrchr.  */
   test_strrchr ();
 
+# if !defined(_AIX)
   /* memrchr.  */
   test_memrchr ();
+# endif
 
   /* rindex - just like strrchr.  */
   test_rindex ();
@@ -1440,8 +1457,10 @@ main (void)
   /* memmove - must work on overlap.  */
   test_memmove ();
 
+# if !defined(_AIX)
   /* mempcpy */
   test_mempcpy ();
+# endif
 
   /* memccpy.  */
   test_memccpy ();
@@ -1478,7 +1497,7 @@ main (void)
   else
     {
       status = EXIT_FAILURE;
-      printf("%Zd errors.\n", errors);
+      printf("%d errors.\n", (int)errors);
     }
 
   return status;
